import {
  Application,
  Infra,
  Either,
  Domain,
  fail,
  ok,
} from '@bitloops/bl-boilerplate-core';
import { Traceable } from '@bitloops/bl-boilerplate-infra-telemetry';
import { Inject } from '@nestjs/common';
import {
  NotificationTemplateReadRepoPortToken,
  UserWriteRepoPortToken,
  StreamingCommandBusToken,
} from '../../../constants';
import { UserWriteRepoPort } from '../../../ports/UserWriteRepoPort';
import { NotificationTemplateReadRepoPort } from '../../../ports/NotificationTemplateReadRepoPort';
import { TodoCompletionsIncrementedDomainEvent } from '../../../domain/events/TodoCompletionsIncrementedDomainEvent';
import { ApplicationErrors } from '../../errors/index';
import { MarketingNotificationDomainService } from '../../../domain/services/MarketingNotificationDomainService';
import { SendEmailCommand } from '../../commands/SendEmailCommand';
export class TodoCompletionsIncrementedDomainEventHandler
  implements Application.IHandleDomainEvent
{
  constructor(
    @Inject(UserWriteRepoPortToken)
    private readonly userRepo: UserWriteRepoPort,
    @Inject(NotificationTemplateReadRepoPortToken)
    private readonly notificationTemplateRepo: NotificationTemplateReadRepoPort,
    @Inject(StreamingCommandBusToken)
    private readonly commandBus: Infra.CommandBus.IStreamCommandBus
  ) {}
  get event() {
    return TodoCompletionsIncrementedDomainEvent;
  }
  get boundedContext(): string {
    return 'marketing';
  }
  @Traceable({
    operation: 'TodoCompletionsIncrementedDomainEventHandler',
    metrics: {
      name: 'TodoCompletionsIncrementedDomainEventHandler',
      category: 'domainEventHandler',
    },
  })
  public async handle(
    event: TodoCompletionsIncrementedDomainEvent
  ): Promise<Either<void, ApplicationErrors.UserNotFoundError>> {
    const user = event.payload;
    const userId = new Domain.UUIDv4(user.aggregateId);
    const userFound = await this.userRepo.getById(userId);
    if (userFound.isFail()) {
      return fail(userFound.value);
    }
    if (!userFound.value) {
      return fail(new ApplicationErrors.UserNotFoundError(user.aggregateId));
    }
    const marketingNotificationService = new MarketingNotificationDomainService(
      this.notificationTemplateRepo
    );
    const emailToBeSentInfo =
      await marketingNotificationService.getNotificationTemplateToBeSent(
        userFound.value
      );
    if (emailToBeSentInfo.isFail()) {
      return fail(emailToBeSentInfo.value);
    }
    if (
      !emailToBeSentInfo.value ||
      !emailToBeSentInfo.value.notificationTemplate
    ) {
      return ok();
    }
    const command = new SendEmailCommand({
      origin: emailToBeSentInfo.value.emailOrigin,
      destination: userFound.value.email.toString(),
      content: emailToBeSentInfo.value.notificationTemplate.template,
    });
    await this.commandBus.publish(command);
    return ok();
  }
}
