import {
  Application,
  Either,
  fail,
  Domain,
  ok,
} from '@bitloops/bl-boilerplate-core';
import { Traceable } from '@bitloops/bl-boilerplate-infra-telemetry';
import { DomainErrors } from '../../domain/errors/index';
import { AddTodoCommand } from '../commands/AddTodoCommand';
import { Inject } from '@nestjs/common';
import { TodoWriteRepoPortToken } from '../../constants';
import { TodoWriteRepoPort } from '../../ports/TodoWriteRepoPort';
import { TitleVO } from '../../domain/TitleVO';
import { UserIdVO } from '../../domain/UserIdVO';
import { TodoEntity } from '../../domain/TodoEntity';
export type AddTodoCommandHandlerResponse = Either<
  string,
  Application.Repo.Errors.Unexpected | DomainErrors.TitleOutOfBoundsError
>;
export class AddTodoCommandHandler
  implements Application.ICommandHandler<AddTodoCommand, string>
{
  constructor(
    @Inject(TodoWriteRepoPortToken)
    private readonly todoRepo: TodoWriteRepoPort
  ) {}
  get command() {
    return AddTodoCommand;
  }
  get boundedContext(): string {
    return 'todo';
  }
  @Traceable({
    operation: 'AddTodoCommandHandler',
    metrics: {
      name: 'AddTodoCommandHandler',
      category: 'commandHandler',
    },
  })
  async execute(
    command: AddTodoCommand
  ): Promise<AddTodoCommandHandlerResponse> {
    const title = TitleVO.create({ title: command.title });
    if (!title.isFail()) {
      const requestUserId = new Domain.UUIDv4(command.metadata.context.userId);
      const userId = UserIdVO.create({ id: requestUserId });
      if (!userId.isFail()) {
        const todo = TodoEntity.create({
          title: title.value,
          completed: false,
          userId: userId.value,
        });
        if (!todo.isFail()) {
          const saveResult = await this.todoRepo.save(todo.value);
          if (saveResult.isFail()) {
            return fail(saveResult.value);
          }
          return ok(todo.value.id.toString());
        } else {
          return fail(todo.value);
        }
      } else {
        return fail(userId.value);
      }
    } else {
      return fail(title.value);
    }
  }
}
