import {
  Application,
  Either,
  Domain,
  fail,
  ok,
} from '@bitloops/bl-boilerplate-core';
import { Traceable } from '@bitloops/bl-boilerplate-infra-telemetry';
import { ApplicationErrors } from '../errors/index';
import { DomainErrors } from '../../domain/errors/index';
import { UncompleteTodoCommand } from '../commands/UncompleteTodoCommand';
import { Inject } from '@nestjs/common';
import { TodoWriteRepoPortToken } from '../../constants';
import { TodoWriteRepoPort } from '../../ports/TodoWriteRepoPort';
export type UncompleteTodoCommandHandlerResponse = Either<
  void,
  | Application.Repo.Errors.Unexpected
  | ApplicationErrors.TodoNotFoundError
  | DomainErrors.TodoAlreadyUncompletedError
>;
export class UncompleteTodoCommandHandler
  implements Application.ICommandHandler<UncompleteTodoCommand, void>
{
  constructor(
    @Inject(TodoWriteRepoPortToken)
    private readonly todoRepo: TodoWriteRepoPort
  ) {}
  get command() {
    return UncompleteTodoCommand;
  }
  get boundedContext(): string {
    return 'todo';
  }
  @Traceable({
    operation: 'UncompleteTodoCommandHandler',
    metrics: {
      name: 'UncompleteTodoCommandHandler',
      category: 'commandHandler',
    },
  })
  async execute(
    command: UncompleteTodoCommand
  ): Promise<UncompleteTodoCommandHandlerResponse> {
    const todoId = new Domain.UUIDv4(command.todoId);
    const todoEntity = await this.todoRepo.getById(todoId);
    if (todoEntity.isFail()) {
      return fail(todoEntity.value);
    }
    if (!todoEntity.value) {
      return fail(new ApplicationErrors.TodoNotFoundError(command.todoId));
    }
    todoEntity.value.uncomplete();
    const updateResult = await this.todoRepo.update(todoEntity.value);
    if (updateResult.isFail()) {
      return fail(updateResult.value);
    }
    return ok();
  }
}
