import { Domain, Either, ok, fail } from '@bitloops/bl-boilerplate-core';
import { TodoProps } from './TodoProps';
import { TodoAddedDomainEvent } from './events/TodoAddedDomainEvent';
import { UserIdVO } from './UserIdVO';
import { TitleVO } from './TitleVO';
import { DomainRules } from './rules/index';
import { TodoCompletedDomainEvent } from './events/TodoCompletedDomainEvent';
import { DomainErrors } from './errors/index';
import { TodoUncompletedDomainEvent } from './events/TodoUncompletedDomainEvent';
import { TodoTitleModifiedDomainEvent } from './events/TodoTitleModifiedDomainEvent';
type TTodoEntityPrimitives = {
  id: string;
  userId: {
    id: string;
  };
  title: {
    title: string;
  };
  completed: boolean;
};
export class TodoEntity extends Domain.Aggregate<TodoProps> {
  private constructor(props: TodoProps) {
    super(props, props.id);
  }
  public static create(props: TodoProps): Either<TodoEntity, never> {
    const todo = new TodoEntity(props);
    const isNew = props.id !== null;
    if (isNew) {
      const event = new TodoAddedDomainEvent({
        aggregateId: todo.id.toString(),
        userId: todo.userId.id.toString(),
        title: todo.title.title,
        completed: todo.completed,
      });
      todo.addDomainEventClass(event);
    }
    return ok(todo);
  }
  get id() {
    return this._id;
  }
  get userId(): UserIdVO {
    return this.props.userId;
  }
  get title(): TitleVO {
    return this.props.title;
  }
  get completed(): boolean {
    return this.props.completed;
  }
  public complete(): Either<void, DomainErrors.TodoAlreadyCompletedError> {
    const res = Domain.applyRules([
      new DomainRules.TodoAlreadyCompletedRule(
        this.props.completed,
        this.props.id.toString()
      ),
    ]);
    if (res) return fail(res);
    this.props.completed = true;
    const event = new TodoCompletedDomainEvent({
      aggregateId: this.props.id.toString(),
      userId: this.props.userId.id.toString(),
      title: this.props.title.title,
      completed: this.props.completed,
    });
    this.addDomainEventClass(event);
    return ok();
  }
  public uncomplete(): Either<void, DomainErrors.TodoAlreadyUncompletedError> {
    const res = Domain.applyRules([
      new DomainRules.TodoAlreadyUncompletedRule(
        this.props.completed,
        this.props.id.toString()
      ),
    ]);
    if (res) return fail(res);
    this.props.completed = false;
    const event = new TodoUncompletedDomainEvent({
      aggregateId: this.props.id.toString(),
      userId: this.props.userId.id.toString(),
      title: this.props.title.title,
      completed: this.props.completed,
    });
    this.addDomainEventClass(event);
    return ok();
  }
  public modifyTitle(title: TitleVO): Either<void, never> {
    this.props.title = title;
    const event = new TodoTitleModifiedDomainEvent({
      aggregateId: this.props.id.toString(),
      userId: this.props.userId.id.toString(),
      title: this.props.title.title,
      completed: this.props.completed,
    });
    this.addDomainEventClass(event);
    return ok();
  }
  public static fromPrimitives(data: TTodoEntityPrimitives): TodoEntity {
    const TodoEntityProps = {
      id: new Domain.UUIDv4(data.id) as Domain.UUIDv4,
      userId: UserIdVO.create({
        id: new Domain.UUIDv4(data.userId.id) as Domain.UUIDv4,
      }).value as UserIdVO,
      title: TitleVO.create({
        title: data.title.title,
      }).value as TitleVO,
      completed: data.completed,
    };
    return new TodoEntity(TodoEntityProps);
  }
  public toPrimitives(): TTodoEntityPrimitives {
    return {
      id: this.id.toString(),
      userId: {
        id: this.props.userId.id.toString(),
      },
      title: {
        title: this.props.title.title,
      },
      completed: this.props.completed,
    };
  }
}
